@@ -13,13 +13,14 @@
  
  Peter Beerli 1996, Seattle
  beerli@genetics.washington.edu
- $Id: data.c,v 1.22 2000/07/26 23:03:20 beerli Exp $
+ $Id: data.c,v 1.30 2001/01/08 17:40:07 beerli Exp $
 
 -------------------------------------------------------*/
 #include <string.h>
 
 #include "migration.h"
 
+extern long number_genomes(char type);
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -27,8 +28,8 @@
 /* prototypes ----------------------------------------- */
 void create_data (data_fmt ** data);
 void get_data (FILE * infile, data_fmt * data, option_fmt * options);
-void print_data (world_fmt * world, data_fmt * data, option_fmt * options);
-void print_data_summary (FILE * file, world_fmt * world);
+void print_data (world_fmt * world, option_fmt * options, data_fmt *data);
+void print_data_summary (FILE * file, world_fmt * world,  option_fmt * options,data_fmt * data);
 void free_datapart (data_fmt * data, option_fmt * options, long locus);
 /*private functions */
 void init_data_structure1 (data_fmt ** data, option_fmt * options);
@@ -37,36 +38,36 @@ void read_sites (data_fmt * data);
 void init_data_structure2 (data_fmt ** data, option_fmt * options, long pop);
 void init_data_structure3 (data_fmt * data);
 void read_popheader (FILE * infile, data_fmt * data, long pop);
-void read_indname (FILE * file, data_fmt * data, long pop, long ind,
-		   long nmlength);
-void read_popdata (FILE * file, data_fmt * data, long pop,
-		   option_fmt * options);
+void read_indname (FILE * file, data_fmt * data, long pop, long ind, long nmlength);
+void read_popdata (FILE * file, data_fmt * data, long pop, option_fmt * options);
 void read_microalleles (FILE * infile, data_fmt * data, long pop, long ind);
 void read_alleles (FILE * infile, data_fmt * data, long pop, long ind);
-long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
-		   long locus, long pop, long ind, long baseread);
-void read_distance_fromfile (FILE * dfile, long tips, long nmlength,
-			     double **m);
-void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options,
-		      long pop, long baseread);
-void print_alleledata (world_fmt * world, data_fmt * data,
-		       option_fmt * options);
-void print_seqdata (world_fmt * world, data_fmt * data);
-void print_header (FILE * outfile, long pop, world_fmt * world);
+long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options, long locus, long pop, long ind, long baseread);
+void read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m);
+void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop, long baseread);
+void print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options);
+void print_seqdata (world_fmt * world, option_fmt * options, data_fmt * data);
+
+void print_header (FILE * outfile, long pop, world_fmt * world, option_fmt * options, data_fmt * data);
 void create_alleles (data_fmt * data);
 void addAllele (data_fmt * data, char s[], long locus, long *z);
 void set_numind (data_fmt * data);
-void print_seq_pop (long locus, long pop, world_fmt * world);
-void print_seq_ind (long locus, long pop, long ind, world_fmt * world);
-void print_locus_head (long locus, world_fmt * world);
+void print_seq_pop (long locus, long pop, world_fmt * world, option_fmt * options, data_fmt * data);
+void print_seq_ind (long locus, long pop, long ind, world_fmt * world, option_fmt * options, data_fmt * data);
+void print_locus_head (long locus, world_fmt * world, option_fmt * options, data_fmt * data);
 void find_delimiter (char *title, char *dlm);
-void read_geofile(data_fmt *data, option_fmt * options, long numpop);
+void read_geofile (data_fmt * data, option_fmt * options, long numpop);
+void read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *uepsites, long datatype);
+void read_uepfile (data_fmt * data, option_fmt * options, long numpop);
+
+
 /*=====================================================*/
 void
 create_data (data_fmt ** data)
 {
   (*data) = (data_fmt *) calloc (1, sizeof (data_fmt));
 }
+
 /*
 void
 init_data (data_fmt * data)
@@ -86,12 +87,13 @@ void
 get_data (FILE * infile, data_fmt * data, option_fmt * options)
 {
   long pop;
-  data->hasghost=FALSE;
+  data->hasghost = FALSE;
   read_header (infile, data, options);
   init_data_structure1 (&data, options);
   switch (options->datatype)
     {
     case 's':
+    case 'f':
       read_sites (data);
       break;
     case 'n':
@@ -116,11 +118,14 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options)
       if (options->progress)
 	fprintf (stdout, "Reading %s ...\n", data->popnames[pop]);
       if (options->writelog)
-	fprintf (options->logfile,"Reading %s ...\n", data->popnames[pop]);
+	fprintf (options->logfile, "Reading %s ...\n", data->popnames[pop]);
       init_data_structure2 (&data, options, pop);
       read_popdata (data->infile, data, pop, options);
     }
-  read_geofile(data, options, data->numpop);
+  read_geofile (data, options, data->numpop);
+#ifdef UEP
+  read_uepfile (data, options, data->numpop);
+#endif
   if (options->progress)
     fprintf (stdout, "\n\n");
   init_data_structure3 (data);
@@ -151,15 +156,12 @@ init_data_structure1 (data_fmt ** data, option_fmt * options)
       (*data)->yy = (char *****) malloc (sizeof (char ****) * (*data)->numpop);
       (*data)->seq = (seqmodel_fmt *) calloc (1, sizeof (seqmodel_fmt));
       (*data)->popnames = (char **) malloc (sizeof (char *) * (*data)->numpop);
-      (*data)->indnames = (char ***)
-	malloc (sizeof (char **) * (*data)->numpop);
+      (*data)->indnames = (char ***) malloc (sizeof (char **) * (*data)->numpop);
       (*data)->numind = (long **) malloc (sizeof (long *) * (*data)->numpop);
       for (pop = 0; pop < (*data)->numpop; pop++)
 	{
-	  (*data)->popnames[pop] = (char *)
-	    calloc (1, sizeof (char) * LINESIZE);
-	  (*data)->numind[pop] = (long *)
-	    malloc (sizeof (long) * (*data)->loci);
+	  (*data)->popnames[pop] = (char *) calloc (1, sizeof (char) * LINESIZE);
+	  (*data)->numind[pop] = (long *) malloc (sizeof (long) * (*data)->loci);
 	}
       (*data)->seq->sites = (long *) calloc (1, sizeof (long) * (*data)->loci);
     }
@@ -176,33 +178,25 @@ init_data_structure2 (data_fmt ** data, option_fmt * options, long pop)
   long ind, locus;
   long indalloc = (*data)->numind[pop][FLOC];
   if (indalloc == 0)
-    indalloc = 2;  
+    indalloc = 2;
   (*data)->yy[pop] = (char ****) malloc (sizeof (char ***) * indalloc);
   (*data)->indnames[pop] = (char **) calloc (1, sizeof (char *) * indalloc);
   for (ind = 0; ind < indalloc; ind++)
     {
-      (*data)->indnames[pop][ind] = (char *)
-	calloc (1, sizeof (char) * (1 + options->nmlength));
-      (*data)->yy[pop][ind] = (char ***)
-	malloc (sizeof (char **) * (*data)->loci);
+      (*data)->indnames[pop][ind] = (char *) calloc (1, sizeof (char) * (1 + options->nmlength));
+      (*data)->yy[pop][ind] = (char ***) malloc (sizeof (char **) * (*data)->loci);
       for (locus = 0; locus < (*data)->loci; locus++)
 	{
-	  if (options->datatype != 's' && options->datatype != 'n' &&
-	      options->datatype != 'u')
+	  if (!strchr(SEQUENCETYPES,options->datatype))
 	    {
-	      (*data)->yy[pop][ind][locus] = (char **)
-		calloc (1, sizeof (char *) * 2);
-	      (*data)->yy[pop][ind][locus][0] =
-		(char *) calloc (1, sizeof (char) * options->allelenmlength);
-	      (*data)->yy[pop][ind][locus][1] =
-		(char *) calloc (1, sizeof (char) * options->allelenmlength);
+	      (*data)->yy[pop][ind][locus] = (char **) calloc (1, sizeof (char *) * 2);
+	      (*data)->yy[pop][ind][locus][0] = (char *) calloc (1, sizeof (char) * options->allelenmlength);
+	      (*data)->yy[pop][ind][locus][1] = (char *) calloc (1, sizeof (char) * options->allelenmlength);
 	    }
 	  else
 	    {
-	      (*data)->yy[pop][ind][locus] = (char **)
-		calloc (1, sizeof (char *));
-	      (*data)->yy[pop][ind][locus][0] =
-		(char *) calloc (1, sizeof (char) * (*data)->seq->sites[locus]);
+	      (*data)->yy[pop][ind][locus] = (char **) calloc (1, sizeof (char *));
+	      (*data)->yy[pop][ind][locus][0] = (char *) calloc (1, sizeof (char) * (*data)->seq->sites[locus]);
 	    }
 	}
     }
@@ -212,18 +206,13 @@ init_data_structure2 (data_fmt ** data, option_fmt * options, long pop)
 void
 free_datapart (data_fmt * data, option_fmt * options, long locus)
 {
-  long ind, pop, genomes;
-  if (options->datatype == 's' || options->datatype == 'n' ||
-      options->datatype == 'u')
-    genomes = 1;
-  else
-    genomes = 2;
+  long ind, pop;
+  long genomes = number_genomes(options->datatype);
   for (pop = 0; pop < data->numpop; pop++)
     {
       for (ind = 0; ind < data->numind[pop][locus] / genomes; ind++)
 	{
-	  if (options->datatype != 's' && options->datatype != 'n' &&
-	      options->datatype != 'u')
+	  if (strchr(SEQUENCETYPES,options->datatype))
 	    {
 	      free (data->yy[pop][ind][locus][0]);
 	      free (data->yy[pop][ind][locus][1]);
@@ -249,8 +238,7 @@ init_data_structure3 (data_fmt * data)
       maxi = 0;
       for (pop = 0; pop < data->numpop; pop++)
 	maxi += data->numind[pop][FLOC] * 2;
-      data->allele[locus] = (allele_fmt *)
-	calloc (1, sizeof (allele_fmt) * maxi);
+      data->allele[locus] = (allele_fmt *) calloc (1, sizeof (allele_fmt) * maxi);
     }
   data->maxalleles = (long *) calloc (1, sizeof (long) * data->loci);
   data->skiploci = (boolean *) calloc (1, sizeof (boolean) * (data->loci + 1));
@@ -267,20 +255,18 @@ read_header (FILE * infile, data_fmt * data, option_fmt * options)
   switch (lowercase (input[0]))
     {
     case 'a':
-      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype,
-	      &(data->numpop), &(data->loci), options->title);
+      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop), &(data->loci), options->title);
       find_delimiter (options->title, &data->dlm);
       break;
     case 'b':
     case 'm':
-      sscanf (input, "%1s%ld%ld%1s%[^\n]", &options->datatype,
-	      &(data->numpop), &(data->loci), &data->dlm, options->title);
+      sscanf (input, "%1s%ld%ld%1s%[^\n]", &options->datatype, &(data->numpop), &(data->loci), &data->dlm, options->title);
       break;
     case 's':
     case 'n':
     case 'u':
-      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype,
-	      &(data->numpop), &(data->loci), options->title);
+    case 'f':
+      sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop), &(data->loci), options->title);
       break;
     case 'g':			/* fall through if a menu change forces to analyze data
 				   instead of using the already sampled genealogies */
@@ -292,20 +278,18 @@ read_header (FILE * infile, data_fmt * data, option_fmt * options)
       switch (options->datatype)
 	{
 	case 'a':
-	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop),
-		  &(data->loci), options->title);
+	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci), options->title);
 	  find_delimiter (options->title, &data->dlm);
 	  break;
 	case 'b':
 	case 'm':
-	  sscanf (input, "%ld%ld%1s%[^\n]", &(data->numpop),
-		  &(data->loci), &(data->dlm), options->title);
+	  sscanf (input, "%ld%ld%1s%[^\n]", &(data->numpop), &(data->loci), &(data->dlm), options->title);
 	  break;
 	case 's':
 	case 'n':
 	case 'u':
-	  sscanf (input, "%ld%ld%[^\n]",
-		  &(data->numpop), &(data->loci), options->title);
+	case 'f':
+	  sscanf (input, "%ld%ld%[^\n]", &(data->numpop), &(data->loci), options->title);
 	  break;
 	default:
 	  usererror ("Datatype is wrong, please use a valid data type!");
@@ -380,8 +364,9 @@ read_popheader (FILE * infile, data_fmt * data, long pop)
     *p = '\0';
   sscanf (input, "%ld%[^\n]", &(data->numind[pop][FLOC]), data->popnames[pop]);
   translate (data->popnames[pop], ' ', '_');
-  if(data->numind[pop][FLOC]==0)
-    data->hasghost=TRUE;
+  translate (data->popnames[pop], '\t', '_');
+  if (data->numind[pop][FLOC] == 0)
+    data->hasghost = TRUE;
 }
 
 void
@@ -414,16 +399,15 @@ read_popdata (FILE * infile, data_fmt * data, long pop, option_fmt * options)
 	case 's':
 	case 'n':
 	case 'u':
+	case 'f':
 	  baseread = read_ind_seq (infile, data, options, locus, pop, ind, 0);
 	  break;
 	default:
-	  usererror
-	    ("Wrong datatype, only the types a, m, s, n\n       (electrophoretic alleles, \n       microsatellite data,\n       sequence data,\n       SNP polymorphism)\n        are allowed.\n");
+	  usererror ("Wrong datatype, only the types a, m, s, n\n       (electrophoretic alleles, \n       microsatellite data,\n       sequence data,\n       SNP polymorphism)\n        are allowed.\n");
 	  break;
 	}
     }
-  if (options->datatype != 's' && options->datatype != 'n' &&
-      options->datatype != 'u')
+  if (!strchr(SEQUENCETYPES,options->datatype))
     return;
   else
     {
@@ -530,16 +514,14 @@ read_alleles (FILE * infile, data_fmt * data, long pop, long ind)
 }
 
 long
-read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
-	      long locus, long pop, long ind, long baseread)
+read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options, long locus, long pop, long ind, long baseread)
 {
   long j;
   char charstate;
   j = (options->interleaved) ? baseread : 0;
   charstate = getc (infile);
   ungetc ((int) charstate, infile);
-  while (j < data->seq->sites[locus]
-	 && !(options->interleaved && charstate == '\n'))
+  while (j < data->seq->sites[locus] && !(options->interleaved && charstate == '\n'))
     {
       charstate = getc (infile);
       if (charstate == '\n')
@@ -549,19 +531,14 @@ read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
 	  else
 	    charstate = ' ';
 	}
-      if (charstate == ' ' || (charstate >= '0' && charstate <= '9')
-	  || charstate == '\\')
+      if (charstate == ' ' || (charstate >= '0' && charstate <= '9') || charstate == '\\')
 	continue;
       charstate = uppercase (charstate);
       //      printf("%c",charstate);
       if ((strchr ("ABCDGHKMNRSTUVWXY?O-", (int) charstate)) == NULL)
 	{
-	  printf
-	    ("ERROR: BAD BASE: %c AT POSITION %5ld OF INDIVIDUUM %3li in POPULATION %ld\n",
-	     charstate, j, ind, pop);
-	  printf
-	    ("Last complete sequences was in population %li, individual %li and locus %li:\n%s",
-	     pop + 1, ind - 1, locus, data->indnames[pop][ind - 1]);
+	  printf ("ERROR: BAD BASE: %c AT POSITION %5ld OF INDIVIDUUM %3li in POPULATION %ld\n", charstate, j, ind, pop);
+	  printf ("Last complete sequences was in population %li, individual %li and locus %li:\n%s", pop + 1, ind - 1, locus, data->indnames[pop][ind - 1]);
 	  for (j = 0; j < data->seq->sites[locus]; j++)
 	    printf ("%c", data->yy[pop][ind - 1][locus][0][j]);
 	  exit (EXIT_FAILURE);
@@ -586,7 +563,7 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m)
       for (i = 0; i < tips; i++)	// of individuals
 	{
 	  //reads header line with
-	  fgets (input, nmlength+1, dfile);
+	  fgets (input, nmlength + 1, dfile);
 	  for (j = 0; j < tips; j++)
 	    {
 	      fscanf (dfile, "%lf", &m[i][j]);
@@ -598,10 +575,58 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m)
     }
 }
 
+#ifdef UEP
+// uep function 
 
 void
-finish_read_seq (FILE * infile, data_fmt * data,
-		 option_fmt * options, long pop, long baseread)
+read_uep_fromfile (FILE * uepfile, long tips, long nmlength, long **uep, long *uepsites, long datatype)
+{
+  char input[LINESIZE];
+  long i, j;
+  long thistips;
+  if (uepfile != NULL)
+    {
+      // assumes that the uepfile is in PHYLIP format
+      // and that all n cells are filled.
+      fgets (input, LINESIZE, uepfile);	//reads header line with
+      // of individuals and uep sites
+      sscanf (input, "%li%li", &thistips, uepsites);
+      if (thistips != tips)
+	error ("UEP datafile and infile are inconsistent!");
+      if(strchr(SEQUENCETYPES,datatype))
+	{
+	  for (i = 0; i < tips; i++)
+	    {
+	      uep[i] = (long *) calloc (*uepsites, sizeof (long));
+	      fgets (input, nmlength + 1, uepfile);  //reads each line
+	      for (j = 0; j < *uepsites; ++j)
+		fscanf (uepfile, "%li", &uep[i][j]);
+	      // reads the last \n from the data matrix
+	      fgets (input, LINESIZE, uepfile);
+	    }
+	}
+      else
+	{
+	  for (i = 0; i < tips; i++)
+	    {
+	      uep[i] = (long *) calloc (*uepsites, sizeof (long));
+	      uep[i+tips] = (long *) calloc (*uepsites, sizeof (long));
+	      fgets (input, nmlength + 1, uepfile);  //reads each line
+	      for (j = 0; j < *uepsites; ++j)
+		fscanf (uepfile, "%li", &uep[i][j]);
+	      // finished reading first allele, no onto the second
+	      for (j = 0; j < *uepsites; ++j)
+		fscanf (uepfile, "%li", &uep[i+tips][j]);
+	      // reads the last \n from the data matrix
+	      fgets (input, LINESIZE, uepfile);
+	    }
+	}
+    }
+}
+#endif
+
+void
+finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options, long pop, long baseread)
 {
 
   long ind, baseread2 = 0, locus = 0;
@@ -611,8 +636,7 @@ finish_read_seq (FILE * infile, data_fmt * data,
 	{
 	  for (ind = 0; ind < data->numind[pop][FLOC]; ind++)
 	    {
-	      baseread2 =
-		read_ind_seq (infile, data, options, locus, pop, ind, baseread);
+	      baseread2 = read_ind_seq (infile, data, options, locus, pop, ind, baseread);
 	    }
 	  baseread = baseread2;
 	}
@@ -631,9 +655,7 @@ finish_read_seq (FILE * infile, data_fmt * data,
 	    {
 	      for (ind = 0; ind < data->numind[pop][FLOC]; ind++)
 		{
-		  baseread2 =
-		    read_ind_seq (infile, data, options, locus, pop, ind,
-				  baseread);
+		  baseread2 = read_ind_seq (infile, data, options, locus, pop, ind, baseread);
 		}
 	      baseread = baseread2;
 	    }
@@ -642,41 +664,42 @@ finish_read_seq (FILE * infile, data_fmt * data,
 }
 
 void
-print_data_summary (FILE * file, world_fmt * world)
+print_data_summary (FILE * file, world_fmt * world, option_fmt * options, data_fmt * data)
 {
   long total = 0;
   long pop;
+  char dstring[LINESIZE];
+
   fprintf (file, "Summary of data:\n");
   fprintf (file, "---------------\n");
-  fprintf (file, "Datatype:                                %20s\n",
-	   world->options->datatype == 'a' ?
-	   "Allelic data" : (world->options->datatype == 's' ?
-			     "Sequence data" :
-			     world->options->datatype == 'n' ||
-			     world->options->datatype == 'u' ?
-			     "SNP data" : "Microsatellite data"));
+  switch(options->datatype)
+    {
+    case 'a': strcpy(dstring,"Allelic data"); break;
+    case 'f': 
+    case 's': strcpy(dstring,"Sequence data"); break;
+    case 'b': 
+    case 'm': strcpy(dstring,"Microsatellite data"); break;
+    case 'n':
+    case 'u': strcpy(dstring,"Allelic data"); break;
+    default:  strcpy(dstring,"Unknown data [ERROR]"); break;
+    }
+  fprintf (file, "Datatype:                                %20s\n", 
+	   dstring);
+  fprintf (file, "Number of loci:                          %20li\n\n", data->loci);
 
-  fprintf (file, "Number of loci:                          %20li\n\n",
-	   world->data->loci);
-
-  fprintf (file,
-	   "Population                                        Individuals\n");
-  fprintf (file,
-	   "-------------------------------------------------------------\n");
+  fprintf (file, "Population                                        Individuals\n");
+  fprintf (file, "-------------------------------------------------------------\n");
   for (pop = 0; pop < world->numpop; pop++)
     {
-      fprintf (file, "%3li %-50.50s %6li\n", pop + 1,
-	       world->data->popnames[pop], world->data->numind[pop][FLOC]);
-      total += world->data->numind[pop][FLOC];
+      fprintf (file, "%3li %-50.50s %6li\n", pop + 1, data->popnames[pop], data->numind[pop][FLOC]);
+      total += data->numind[pop][FLOC];
     }
-  fprintf (file,
-	   "Total of all populations                               %6li\n\n",
-	   total);
+  fprintf (file, "Total of all populations                               %6li\n\n", total);
   fflush (file);
 }
 
 void
-print_data (world_fmt * world, data_fmt * data, option_fmt * options)
+print_data (world_fmt * world, option_fmt * options, data_fmt * data)
 {
   if (options->printdata)
     {
@@ -690,7 +713,8 @@ print_data (world_fmt * world, data_fmt * data, option_fmt * options)
 	case 's':
 	case 'n':
 	case 'u':
-	  print_seqdata (world, data);
+	case 'f':
+	  print_seqdata (world, options, data);
 	  break;
 	}
     }
@@ -702,18 +726,14 @@ print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
   long i, pop, ind, locus, mult80;
   for (pop = 0; pop < data->numpop; pop++)
     {
-      print_header (world->outfile, pop, world);
+      print_header (world->outfile, pop, world, options, data);
       for (ind = 0; ind < data->numind[pop][FLOC]; ind++)
 	{
-	  fprintf (world->outfile, "%-*.*s ",
-		   (int) options->nmlength, (int) options->nmlength,
-		   data->indnames[pop][ind]);
+	  fprintf (world->outfile, "%-*.*s ", (int) options->nmlength, (int) options->nmlength, data->indnames[pop][ind]);
 	  mult80 = options->nmlength;
 	  for (locus = 0; locus < data->loci; locus++)
 	    {
-	      mult80 +=
-		1 + strlen (data->yy[pop][ind][locus][0]) +
-		strlen (data->yy[pop][ind][locus][1]);
+	      mult80 += 1 + strlen (data->yy[pop][ind][locus][0]) + strlen (data->yy[pop][ind][locus][1]);
 	      if (mult80 >= 80)
 		{
 		  mult80 = 0;
@@ -721,9 +741,7 @@ print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
 		  for (i = 0; i < options->nmlength; i++)
 		    fputc (' ', world->outfile);
 		}
-	      fprintf (world->outfile, " %s.%-s",
-		       data->yy[pop][ind][locus][0],
-		       data->yy[pop][ind][locus][1]);
+	      fprintf (world->outfile, " %s.%-s", data->yy[pop][ind][locus][0], data->yy[pop][ind][locus][1]);
 	    }
 	  fprintf (world->outfile, "\n");
 	}
@@ -734,50 +752,48 @@ print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
 }
 
 void
-print_seqdata (world_fmt * world, data_fmt * data)
+print_seqdata (world_fmt * world, option_fmt * options, data_fmt * data)
 {
   long pop, locus;
   for (pop = 0; pop < data->numpop; pop++)
     {
-      print_header (world->outfile, pop, world);
+      print_header (world->outfile, pop, world, options, data);
       for (locus = 0; locus < data->loci; locus++)
 	{
-	  print_locus_head (locus, world);
-	  print_seq_pop (locus, pop, world);
+	  print_locus_head (locus, world, options, data);
+	  print_seq_pop (locus, pop, world, options, data);
 	}
     }
   fflush (world->outfile);
 }
 
 void
-print_header (FILE * outfile, long pop, world_fmt * world)
+print_header (FILE * outfile, long pop, world_fmt * world, option_fmt * options, data_fmt * data)
 {
   long i;
   long locus, mult80 = 80;
   char input[LINESIZE];
-  fprintf (outfile, "\n%-s", world->data->popnames[pop]);
-  for (i = 0; i < (long) (80 - strlen (world->data->popnames[pop])); i++)
+  fprintf (outfile, "\n%-s", data->popnames[pop]);
+  for (i = 0; i < (long) (80 - strlen (data->popnames[pop])); i++)
     fputc ('-', outfile);
   fprintf (outfile, "\n\n");
-  if (world->options->datatype != 's' &&
-      world->options->datatype != 'n' && world->options->datatype != 'u')
+  if (!strchr(SEQUENCETYPES,options->datatype))
     {
-      fprintf (outfile, "%-s  ", (world->data->loci == 1 ? "locus" : "loci "));
-      for (i = 0; i < (world->options->nmlength - 6); i++)
+      fprintf (outfile, "%-s  ", (data->loci == 1 ? "locus" : "loci "));
+      for (i = 0; i < (options->nmlength - 6); i++)
 	fputc (' ', outfile);
-      for (locus = 0; locus < world->data->loci; locus++)
+      for (locus = 0; locus < data->loci; locus++)
 	{
-	  if (locus * 4 + world->options->nmlength > mult80)
+	  if (locus * 4 + options->nmlength > mult80)
 	    {
 	      mult80 += 80;
 	      fprintf (outfile, "\n");
-	      for (i = 0; i < world->options->nmlength; i++)
+	      for (i = 0; i < options->nmlength; i++)
 		fputc (' ', outfile);
 	    }
 	  fprintf (outfile, "  %2li", locus + 1);
 	}
-      fprintf (outfile, "\n%-s\n",
-	       strncpy (input, "indiv.", world->options->nmlength));
+      fprintf (outfile, "\n%-s\n", strncpy (input, "indiv.", options->nmlength));
     }
 }
 
@@ -819,8 +835,7 @@ void
 addAllele (data_fmt * data, char s[], long locus, long *z)
 {
   long found = 0;
-  while ((data->allele[locus][found++][0] != '\0')
-	 && (strcmp (s, data->allele[locus][found - 1])))
+  while ((data->allele[locus][found++][0] != '\0') && (strcmp (s, data->allele[locus][found - 1])))
     ;
   if (found > (*z))
     {
@@ -844,31 +859,27 @@ set_numind (data_fmt * data)
 
 
 void
-print_seq_pop (long locus, long pop, world_fmt * world)
+print_seq_pop (long locus, long pop, world_fmt * world, option_fmt * options, data_fmt * data)
 {
   long ind;
-  for (ind = 0; ind < world->data->numind[pop][locus]; ind++)
+  for (ind = 0; ind < data->numind[pop][locus]; ind++)
     {
-      print_seq_ind (locus, pop, ind, world);
+      print_seq_ind (locus, pop, ind, world, options, data);
     }
 }
 
 void
-print_seq_ind (long locus, long pop, long ind, world_fmt * world)
+print_seq_ind (long locus, long pop, long ind, world_fmt * world, option_fmt * options, data_fmt * data)
 {
   long site;
   char blank[2] = " ";
-  fprintf (world->outfile, "%-*.*s", (int) world->options->nmlength,
-	   (int) world->options->nmlength, world->data->indnames[pop][ind]);
-  fprintf (world->outfile, " %c", world->data->yy[pop][ind][locus][0][0]);
-  for (site = 1; site < world->data->seq->sites[locus]; site++)
+  fprintf (world->outfile, "%-*.*s", (int) options->nmlength, (int) options->nmlength, data->indnames[pop][ind]);
+  fprintf (world->outfile, " %c", data->yy[pop][ind][locus][0][0]);
+  for (site = 1; site < data->seq->sites[locus]; site++)
     {
       if ((site) % 60 == 0)
 	{
-	  fprintf (world->outfile, "\n%-*.*s %c",
-		   (int) world->options->nmlength,
-		   (int) world->options->nmlength, blank,
-		   world->data->yy[pop][ind][locus][0][site]);
+	  fprintf (world->outfile, "\n%-*.*s %c", (int) options->nmlength, (int) options->nmlength, blank, data->yy[pop][ind][locus][0][site]);
 	}
       else
 	{
@@ -876,8 +887,7 @@ print_seq_ind (long locus, long pop, long ind, world_fmt * world)
 	    {
 	      fprintf (world->outfile, " ");
 	    }
-	  fprintf (world->outfile, "%c",
-		   world->data->yy[pop][ind][locus][0][site]);
+	  fprintf (world->outfile, "%c", data->yy[pop][ind][locus][0][site]);
 	}
     }
   fprintf (world->outfile, "\n");
@@ -885,25 +895,22 @@ print_seq_ind (long locus, long pop, long ind, world_fmt * world)
 
 
 void
-print_locus_head (long locus, world_fmt * world)
+print_locus_head (long locus, world_fmt * world, option_fmt * options, data_fmt * data)
 {
   char *head;
-  head =
-    (char *) calloc (1, sizeof (char) * MAX (10, world->options->nmlength));
+  head = (char *) calloc (1, sizeof (char) * MAX (10, options->nmlength));
   sprintf (head, "Locus %li", locus);
-  fprintf (world->outfile, "%-*.*s --------10 --------20 --------30",
-	   (int) world->options->nmlength,
-	   (int) world->options->nmlength, head);
+  fprintf (world->outfile, "%-*.*s --------10 --------20 --------30", (int) options->nmlength, (int) options->nmlength, head);
   fprintf (world->outfile, " --------40 --------50 --------60\n");
 
   free (head);
 }
 
 void
-read_geofile(data_fmt *data, option_fmt * options, long numpop)
+read_geofile (data_fmt * data, option_fmt * options, long numpop)
 {
   long i, j, pop;
-  long numpop2 = numpop*numpop;
+  long numpop2 = numpop * numpop;
   data->geo = (double *) calloc (1, sizeof (double) * numpop2);
   data->lgeo = (double *) calloc (1, sizeof (double) * numpop2);
   if (!options->geo)
@@ -914,17 +921,35 @@ read_geofile(data_fmt *data, option_fmt * options, long numpop)
   else
     {
       data->ogeo = (double **) calloc (1, sizeof (double *) * numpop);
-      data->ogeo[0] = (double *) calloc (1, sizeof (double) *
-					numpop2);
+      data->ogeo[0] = (double *) calloc (1, sizeof (double) * numpop2);
       for (pop = 1; pop < numpop; pop++)
 	data->ogeo[pop] = data->ogeo[0] + numpop * pop;
-      read_distance_fromfile (data->geofile, numpop,
-			      options->nmlength, data->ogeo);
+      read_distance_fromfile (data->geofile, numpop, options->nmlength, data->ogeo);
       for (i = 0; i < numpop; i++)
-	for (j = i+1; j < numpop; j++)
+	for (j = i + 1; j < numpop; j++)
 	  {
-	    data->geo[mm2m(i,j,numpop)] = data->geo[mm2m(j,i,numpop)] = data->ogeo[i][j];
-	    data->lgeo[mm2m(i,j,numpop)] = data->lgeo[mm2m(j,i,numpop)] = data->ogeo[i][j]>0.0 ? log(data->ogeo[i][j]) : -DBL_MAX;;
+	    data->geo[mm2m (i, j, numpop)] = data->geo[mm2m (j, i, numpop)] = data->ogeo[i][j];
+	    data->lgeo[mm2m (i, j, numpop)] = data->lgeo[mm2m (j, i, numpop)] = data->ogeo[i][j] > 0.0 ? log (data->ogeo[i][j]) : -DBL_MAX;;
 	  }
     }
 }
+
+#ifdef UEP
+void
+read_uepfile (data_fmt * data, option_fmt * options, long numpop)
+{
+  long i;
+  long sumtips = 0;
+
+  if (!options->uep)
+    return;
+
+  for (i = 0; i < numpop; ++i)
+    sumtips += data->numind[i][FLOC];
+  data->uep = (long **) calloc (number_genomes(options->datatype) * sumtips, 
+				sizeof (long *));
+  read_uep_fromfile (data->uepfile, sumtips, options->nmlength, data->uep, &data->uepsites, options->datatype);
+}
+
+#endif
+
