@@ -11,13 +11,16 @@
 
  Peter Beerli 1996, Seattle
  beerli@genetics.washington.edu
- $Id: tools.c,v 1.28 2000/07/26 23:58:54 beerli Exp $
+ $Id: tools.c,v 1.38 2001/03/09 16:29:19 beerli Exp $
 
 -------------------------------------------------------*/
 #include "migration.h"
+#include "data.h"
 #include "world.h"
 #include "random.h"
 #include "broyden.h"
+#include "migrate_mpi.h"
+#include "options.h"
 
 #include <math.h>
 #ifdef DMALLOC_FUNC_CHECK
@@ -30,15 +33,13 @@ node *crawlback (const node * theNode);
 /*node *crawl(node * theNode); */
 node *showtop (node * theNode);
 void adjust_time (node * theNode, double tyme);
-void insert_migr_node (world_fmt * world, node * up, node * down,
-		       migr_table_fmt * migr_table, long *migr_table_counter);
+void insert_migr_node (world_fmt * world, node * up, node * down, migr_table_fmt * migr_table, long *migr_table_counter);
 void children (node * mother, node ** brother, node ** sister);
 /* math tools */
 double incompletegamma (double x, double alpha);
 double polygamma (long n, double z);
 void invert_matrix (double **a, long nsize);
-boolean nrcheck (double **m, double **tm, double *v, long nrows, double *r1,
-		 double *r2, boolean do_newton);
+boolean nrcheck (double **m, double **tm, double *v, long nrows, double *r1, double *r2, boolean do_newton);
 double rannor (double mean, double sd);
 char lowercase (char c);
 char uppercase (char c);
@@ -50,18 +51,26 @@ void calc_gamma (double alpha, double *gama, long categs);
 
 double mylgamma (double z);
 
+double logfac (long n);
+
+/* vector initialization */
+void doublevec1d(double **v, long size);
+void doublevec2d(double ***v, long size1, long size2);
+void free_doublevec2d(double **v);
+void setdoublevec1d(double **v, double *w, long size);
+void add_vector(double *result, double *v, long size);
 
 /*filemanipulation */
 void init_files (world_fmt * world, data_fmt * data, option_fmt * options);
 void exit_files (world_fmt * world, data_fmt * data, option_fmt * options);
-void openfile (FILE ** fp, char *filename, char *mode, const char *applic,
-	       char *perm);
-void read_savesum (world_fmt * world, option_fmt * options, data_fmt * data);
+void openfile (FILE ** fp, char *filename, char *mode, const char *applic, char *perm);
+long read_savesum (world_fmt * world, option_fmt * options, data_fmt * data);
 void write_savesum (world_fmt * world);
 
 /* string manipulation */
 void translate (char *text, char from, char to);
 long count_words (char *text);
+void print_line (FILE * outfile, char c, long nn, long flag);
 
 /* time reporting */
 void get_time (char *nowstr, char ts[]);
@@ -90,12 +99,85 @@ void lu_decomp (double **m, long *indeks, long nrows);
 void lu_substitution (double **m, long *indeks, double *v, long nrows);
 double d1mach (long i);
 long i1mach (long i);
-int dpsifn (double *x, long *n, long kode, long m, double *ans, long *nz,
-	    long *ierr);
+int dpsifn (double *x, long *n, long kode, long m, double *ans, long *nz, long *ierr);
 double find_chi (long df, double prob);
 double probchi (long df, double chi);
 double chisquare (long df, double alpha);
 
+/* vector initialization */
+void doublevec1d(double **v, long size)
+{
+  *v = (double *) calloc(size,sizeof(double));
+}
+
+void doublevec2d(double ***v, long size1, long size2)
+{
+  long i;
+  *v = (double **) calloc(size1,sizeof(double *));
+  (*v)[0] = (double *) calloc(size1*size2,sizeof(double));
+  for(i=1;i<size1;i++)
+    {
+      (*v)[i] = (*v)[0] + size1 * i;
+    }
+}
+
+void free_doublevec2d(double **v)
+{
+  free(v[0]);
+  free(v);
+}
+
+void setdoublevec1d(double **v, double *w, long size)
+{
+  doublevec1d(v, size);
+  memcpy(*v, w, sizeof(double) * size);
+}
+
+void add_vector(double *result, double *v, long size)
+{
+  long i;
+  for(i=0; i<size; i++)
+    result[i] += v[i];
+}
+
+
+double
+logfac (long n)
+{
+  /* log(n!) values were calculated with Mathematica 
+     with a precision of 30 digits */
+  switch (n)
+    {
+    case 0:
+      return 0.;
+    case 1:
+      return 0.;
+    case 2:
+      return 0.693147180559945309417232121458;
+    case 3:
+      return 1.791759469228055000812477358381;
+    case 4:
+      return 3.1780538303479456196469416013;
+    case 5:
+      return 4.78749174278204599424770093452;
+    case 6:
+      return 6.5792512120101009950601782929;
+    case 7:
+      return 8.52516136106541430016553103635;
+    case 8:
+      return 10.60460290274525022841722740072;
+    case 9:
+      return 12.80182748008146961120771787457;
+    case 10:
+      return 15.10441257307551529522570932925;
+    case 11:
+      return 17.50230784587388583928765290722;
+    case 12:
+      return 19.98721449566188614951736238706;
+    default:
+      return LGAMMA (n + 1.);
+    }
+}
 
 
 /*FILEMANIPULATION======================================================= */
@@ -103,38 +185,45 @@ void
 init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 {
   openfile (&data->infile, options->infilename, "r+", appl, NULL);
-  openfile (&world->outfile, options->outfilename, "w+", appl, NULL);
-  if(options->writelog)
-    openfile (&options->logfile, options->logfilename, "w+", appl, NULL);
+  if(myID==MASTER)
+    {
+      openfile (&world->outfile, options->outfilename, "w+", appl, NULL);
+      if (options->treeprint > 0)
+	openfile (&world->treefile, options->treefilename, "w+", appl, NULL);
+      if (options->mighist)
+	openfile (&world->mighistfile, options->mighistfilename, "w+", appl, NULL);
+      if (options->writelog)
+	openfile (&options->logfile, options->logfilename, "w+", appl, NULL);
+      if (options->writesum)
+	openfile (&data->sumfile, options->sumfilename, "w+", appl, NULL);
+      if (options->plot)
+	{
+	  switch (options->plotmethod)
+	    {
+	    case 0:
+	      openfile (&world->mathfile, options->mathfilename, "w+", appl, NULL);
+	      break;
+	    default:		/*e.g. 0 this create just the plots in outfile */
+	      break;
+	    }
+	}
+    }
   if (options->usertree)
     openfile (&data->utreefile, options->utreefilename, "r+", appl, NULL);
   if (options->readsum)
     openfile (&data->sumfile, options->sumfilename, "r+", appl, NULL);
-  if (options->writesum)
-    openfile (&data->sumfile, options->sumfilename, "w+", appl, NULL);
   if (options->weights)
     openfile (&data->weightfile, options->weightfilename, "r+", appl, NULL);
   if (options->categs > 1)
     openfile (&data->catfile, options->catfilename, "r+", appl, NULL);
-  if (options->treeprint > 0)
-    openfile (&world->treefile, options->treefilename, "w+", appl, NULL);
-  if (options->dist)
+   if (options->dist)
     openfile (&data->distfile, options->distfilename, "r+", appl, NULL);
   if (options->geo)
     openfile (&data->geofile, options->geofilename, "r+", appl, NULL);
-  if (options->mighist)
-    openfile (&world->mighistfile, options->mighistfilename, "w+", appl, NULL);
-  if (options->plot)
-    {
-      switch (options->plotmethod)
-	{
-	case 0:
-	  openfile (&world->mathfile, options->mathfilename, "w+", appl, NULL);
-	  break;
-	default:		/*e.g. 0 this create just the plots in outfile */
-	  break;
-	}
-    }
+#ifdef UEP
+  if (options->uep)
+    openfile (&data->uepfile, options->uepfilename, "r+", appl, NULL);
+#endif
 }
 
 void
@@ -142,8 +231,8 @@ exit_files (world_fmt * world, data_fmt * data, option_fmt * options)
 {
   FClose (data->infile);
   FClose (world->outfile);
-  if(options->writelog)
-    FClose(options->logfile);
+  if (options->writelog)
+    FClose (options->logfile);
   if (options->weights)
     FClose (data->weightfile);
   if (options->categs > 1)
@@ -160,6 +249,11 @@ exit_files (world_fmt * world, data_fmt * data, option_fmt * options)
     FClose (world->mathfile);
   if (options->writesum || options->readsum)
     FClose (data->sumfile);
+#ifdef UEP
+  if (options->uep)
+    FClose (data->uepfile);
+#endif
+
 }
 
 /* string manipulation ================================== */
@@ -260,8 +354,7 @@ print_llike (double llike, char *strllike)
 }
 
 void
-openfile (FILE ** fp, char *filename, char *mode, const char *application,
-	  char *perm)
+openfile (FILE ** fp, char *filename, char *mode, const char *application, char *perm)
 {
   int trials = 0;
   FILE *of = NULL;
@@ -310,115 +403,113 @@ openfile (FILE ** fp, char *filename, char *mode, const char *application,
 
 }
 
-void
+long
 read_savesum (world_fmt * world, option_fmt * options, data_fmt * data)
 {
+  long Gmax=0;
   FILE *sumfile = data->sumfile;
   timearchive_fmt **ta;
-  long nrep=0;
+  long nrep = 0;
   long l, i, j, r;
   char input[1024];
   long hits;
   long tmp;
-  long z,zz, jj;
+  long z, zz, jj;
   double tmpm;
-  boolean  newstyle=TRUE;
+  boolean newstyle = TRUE;
 
   fgets (input, 1024, sumfile);
   if (!strncmp ("# begin genealogy-summary file of migrate", input, 41))
     {
-      fgets(input,1024,sumfile);
-      if(input[0]=='#')
+      fgets (input, 1024, sumfile);
+      if (input[0] == '#')
 	{
-	  newstyle=TRUE;
-	  fgets(input,1024,sumfile);
+	  newstyle = TRUE;
+	  fgets (input, 1024, sumfile);
 	}
       else
-	newstyle=FALSE;
-      hits = sscanf (input, "%li %li %li %li %li",
-	      &world->loci, &world->numpop, &world->numpop2, &tmp,
-		     &options->replicatenum);
-      if(hits!=5)
+	newstyle = FALSE;
+      hits = sscanf (input, "%li %li %li %li %li", &world->loci, &world->numpop, &world->numpop2, &tmp, &options->replicatenum);
+      if (hits != 5)
 	{
-	  nrep=1;
-	  hits = sscanf (input, "%li %li %li",
-			 &world->loci, &world->numpop, &world->numpop2);
+	  nrep = 1;
+	  hits = sscanf (input, "%li %li %li", &world->loci, &world->numpop, &world->numpop2);
 	  options->replicate = FALSE;
 	  options->replicatenum = 0;
 	}
       else
 	{
-	  nrep= options->replicatenum;
-	  if(nrep==0)
-	    nrep=1;
+	  nrep = options->replicatenum;
+	  if (nrep == 0)
+	    nrep = 1;
 	  options->replicate = (boolean) tmp;
 	}
       data->loci = world->loci;
-      data->skiploci = (boolean *)
-	realloc (data->skiploci, sizeof (boolean) * (data->loci + 1));
+      data->skiploci = (boolean *) realloc (data->skiploci, sizeof (boolean) * (data->loci + 1));
       memset (data->skiploci, 0, sizeof (boolean) * (data->loci + 1));
+      read_geofile (data, options, world->numpop);
+      data->numpop = world->numpop;
+      set_plot (options);
       init_world (world, data, options);
-      ta = world->atl; 
+      ta = world->atl;
       for (l = 0; l < world->loci; l++)
 	for (r = 0; r < nrep; r++)
-	{
-	  fgets (input, 1024, sumfile);
-	  fscanf (sumfile, "%li %li %li\n", &ta[r][l].T,
-		  &ta[r][l].numpop, &ta[r][l].sumtips);
-	  ta[r][l].allocT = 0.;
-	  increase_timearchive (world, l, ta[r][l].T, world->numpop,r);
-	  fscanf (sumfile, "%lg %lg %lg\n", &ta[r][l].param_like,
-		  &ta[r][l].thb, &ta[r][l].alpha);
-	  for (i = 0; i < world->atl[r][l].T; i++)
-	    {
-	      fscanf (sumfile, "%li ", &ta[r][l].tl[i].copies);
-	      for (j = 0; j < world->numpop; j++)
-		{
-		  fscanf (sumfile, "%lf %lf %li\n",
-			  &ta[r][l].tl[i].km[j], &ta[r][l].tl[i].kt[j],
-			  &ta[r][l].tl[i].p[j]);
-		}
-	      if(newstyle)
-		{
-		  for (j = 0; j < world->numpop2; j++)
-		    {
-		      fscanf (sumfile, "%lf ", &ta[r][l].tl[i].mindex[j]);
-		    }
-		}
-	      else
-		{
-		  z=0;
-		  zz = world->numpop;
-		  for (j = 0; j < world->numpop; j++)
-		    {
-		      for(jj=0; jj < world->numpop;jj++)
-			{
-			  fscanf (sumfile, "%lf ", &tmpm);
-			  if(j==jj)
-			    ta[r][l].tl[i].mindex[z++] = tmpm;
-			  else
-			    ta[r][l].tl[i].mindex[zz++] = tmpm;
-			}
-		    }
-		}
-	    }
-	  for (i = 0; i < world->numpop2; i++)
-	    {
-	      fscanf (sumfile, "%lg %lg\n", &ta[r][l].param[i], &ta[r][l].param0[i]);
-	    }
-	  log_param0 (ta[r][l].param0, ta[r][l].lparam0, world->numpop2);
-	  //for(i=0;i<ta[l].T;i++)
-	  //  {
-	  //    fscanf(sumfile,"%lg ",&ta[l].likelihood[i]);
-	  //  }
-	  //      fgets(input,1024,sumfile);
-	  fscanf (sumfile, "%li %lg\n", &ta[r][l].trials, &ta[r][l].normd);
-	}
+	  {
+	    fgets (input, 1024, sumfile);
+	    fscanf (sumfile, "%li %li %li\n", &ta[r][l].T, &ta[r][l].numpop, &ta[r][l].sumtips);
+	    Gmax = (ta[r][l].T > Gmax) ? ta[r][l].T : Gmax;
+	    ta[r][l].allocT = 0;
+	    increase_timearchive (world, l, ta[r][l].T, world->numpop, r);
+	    fscanf (sumfile, "%lg %lg %lg\n", &ta[r][l].param_like, &ta[r][l].thb, &ta[r][l].alpha);
+	    for (i = 0; i < world->atl[r][l].T; i++)
+	      {
+		fscanf (sumfile, "%li ", &ta[r][l].tl[i].copies);
+		for (j = 0; j < world->numpop; j++)
+		  {
+		    fscanf (sumfile, "%lf %lf %li\n", &ta[r][l].tl[i].km[j], &ta[r][l].tl[i].kt[j], &ta[r][l].tl[i].p[j]);
+		  }
+		if (newstyle)
+		  {
+		    for (j = 0; j < world->numpop2; j++)
+		      {
+			fscanf (sumfile, "%lf ", &ta[r][l].tl[i].mindex[j]);
+		      }
+		  }
+		else
+		  {
+		    z = 0;
+		    zz = world->numpop;
+		    for (j = 0; j < world->numpop; j++)
+		      {
+			for (jj = 0; jj < world->numpop; jj++)
+			  {
+			    fscanf (sumfile, "%lf ", &tmpm);
+			    if (j == jj)
+			      ta[r][l].tl[i].mindex[z++] = tmpm;
+			    else
+			      ta[r][l].tl[i].mindex[zz++] = tmpm;
+			  }
+		      }
+		  }
+	      }
+	    for (i = 0; i < world->numpop2; i++)
+	      {
+		fscanf (sumfile, "%lg %lg\n", &ta[r][l].param[i], &ta[r][l].param0[i]);
+	      }
+	    log_param0 (ta[r][l].param0, ta[r][l].lparam0, world->numpop2);
+	    //for(i=0;i<ta[l].T;i++)
+	    //  {
+	    //    fscanf(sumfile,"%lg ",&ta[l].likelihood[i]);
+	    //  }
+	    //      fgets(input,1024,sumfile);
+	    fscanf (sumfile, "%li %lg\n", &ta[r][l].trials, &ta[r][l].normd);
+	  }
     }
   else
     {
       error ("This is not a genealogy-summary file for MIGRATE\n");
     }
+  return Gmax;
 }
 
 void
@@ -429,37 +520,29 @@ write_savesum (world_fmt * world)
   timearchive_fmt **ta = world->atl;
 
   long l, i, j;
-  if(world->options->replicate)
+  if (world->options->replicate)
     {
-      if(world->options->replicatenum==0)
+      if (world->options->replicatenum == 0)
 	repmax = world->options->lchains;
       else
 	repmax = world->options->replicatenum;
     }
   else
-    repmax=1;
-  fprintf (sumfile, "# begin genealogy-summary file of migrate %s ------\n#\n",
-	   MIGRATEVERSION);
-  fprintf (sumfile, "%li %li %li %li %li\n",
-	   world->loci, world->numpop, world->numpop2,
-	   (long) world->options->replicate, repmax);
+    repmax = 1;
+  fprintf (sumfile, "# begin genealogy-summary file of migrate %s ------\n#\n", MIGRATEVERSION);
+  fprintf (sumfile, "%li %li %li %li %li\n", world->loci, world->numpop, world->numpop2, (long) world->options->replicate, repmax);
   for (l = 0; l < world->loci; l++)
     {
-      for(r=0;r<repmax;r++)
-	{ 
-	  fprintf (sumfile,
-		   "%li %li ####### locus %li, replicate %li ################\n%li %li %li\n", l, r, l, r,
-		   ta[r][l].T, ta[r][l].numpop, ta[r][l].sumtips);
-	  fprintf (sumfile, "%20.20g %20.20g %20.20g\n", ta[r][l].param_like,
-		   ta[r][l].thb, ta[r][l].alpha);
+      for (r = 0; r < repmax; r++)
+	{
+	  fprintf (sumfile, "%li %li ####### locus %li, replicate %li ################\n%li %li %li\n", l, r, l, r, ta[r][l].T, ta[r][l].numpop, ta[r][l].sumtips);
+	  fprintf (sumfile, "%20.20g %20.20g %20.20g\n", ta[r][l].param_like, ta[r][l].thb, ta[r][l].alpha);
 	  for (i = 0; i < ta[r][l].T; i++)
 	    {
 	      fprintf (sumfile, "%li ", ta[r][l].tl[i].copies);
 	      for (j = 0; j < world->numpop; j++)
 		{
-		  fprintf (sumfile, "%20.20f %20.20f %li\n",
-			   ta[r][l].tl[i].km[j], ta[r][l].tl[i].kt[j], 
-			   ta[r][l].tl[i].p[j]);
+		  fprintf (sumfile, "%20.20f %20.20f %li\n", ta[r][l].tl[i].km[j], ta[r][l].tl[i].kt[j], ta[r][l].tl[i].p[j]);
 		}
 	      for (j = 0; j < world->numpop2; j++)
 		{
@@ -470,8 +553,7 @@ write_savesum (world_fmt * world)
 	    }
 	  for (i = 0; i < world->numpop2; i++)
 	    {
-	      fprintf (sumfile, "%20.20e %20.20e\n", ta[r][l].param[i],
-		       ta[r][l].param0[i]);
+	      fprintf (sumfile, "%20.20e %20.20e\n", ta[r][l].param[i], ta[r][l].param0[i]);
 	    }
 	  //  for(i=0;i<ta[l].T;i++)
 	  //        {
@@ -481,11 +563,10 @@ write_savesum (world_fmt * world)
 	  fprintf (sumfile, "%li %20.20e\n", ta[r][l].trials, ta[r][l].normd);
 	}
     }
-  fprintf (sumfile, "# end genealogy-summary file of migrate %s ------\n",
-	   MIGRATEVERSION);
+  fprintf (sumfile, "# end genealogy-summary file of migrate %s ------\n", MIGRATEVERSION);
   fflush (sumfile);
 }
-  
+
 
 /*=======================================================*/
 
@@ -592,8 +673,7 @@ adjust_time (node * theNode, double tyme)
 }
 
 void
-insert_migr_node (world_fmt * world, node * up, node * down,
-		  migr_table_fmt * migr_table, long *migr_table_counter)
+insert_migr_node (world_fmt * world, node * up, node * down, migr_table_fmt * migr_table, long *migr_table_counter)
 {
   long i, panic;
   node *tmp, *tmp2, *oldNode, *oldNode2, *theNode;
@@ -601,15 +681,13 @@ insert_migr_node (world_fmt * world, node * up, node * down,
     error ("up has to be a top-node");
   theNode = showtop (up)->back;
   if (*migr_table_counter > 0 && up->tyme > migr_table[0].time)
-    error
-      ("insert_migr_node: the first migration node has a wrong time for up");
+    error ("insert_migr_node: the first migration node has a wrong time for up");
   if (migr_table[(*migr_table_counter) - 1].from != down->actualpop)
     {
       error ("this should never happen -> wrong choice of nodes\n");
       (*migr_table_counter)--;
     }
-  if (((*migr_table_counter) > 0)
-      && (migr_table[(*migr_table_counter) - 1].from != down->actualpop))
+  if (((*migr_table_counter) > 0) && (migr_table[(*migr_table_counter) - 1].from != down->actualpop))
     error ("problem catched in inser_migr_table");
   for (i = 0; i < (*migr_table_counter); i++)
     {
@@ -707,7 +785,7 @@ mylgamma (double z)
   double tmp;
   if (z <= 0.)
     {
-      return DBL_MAX;/*this will kill the receiving calculation */
+      return DBL_MAX;		/*this will kill the receiving calculation */
     }
   result = 0.;
   tmp = z + 7.;
@@ -731,22 +809,20 @@ double
 incompletegamma (double x, double alpha)
 {
   double gama, d_1, d_2, d_3;
-  /*static*/ double a, b, c, an, rn;
-  /*static*/ double pn1, pn2, pn3, pn4, pn5, pn6, arg;
+  /*static */ double a, b, c, an, rn;
+  /*static */ double pn1, pn2, pn3, pn4, pn5, pn6, arg;
 
   gama = 0.;
   /*  Check that we have valid values for X and P */
   if (alpha <= 0. || x < 0.)
-      error ("failed in imcompletegamma(): wrong alpha or x\n");
-  if (x == 0.)
-      return gama;
+    error ("failed in imcompletegamma(): wrong alpha or x\n");
+  if (fabs(x) < DBL_EPSILON)
+    return gama;
 
   /*  Use a normal approximation if P > PLIMIT */
   if (alpha > 1e3)
     {
-      pn1 =
-	sqrt (alpha) * 3. * (pow (x / alpha, (1. / 3.)) + 
-			     1. / (alpha * 9.) - 1.);
+      pn1 = sqrt (alpha) * 3. * (pow (x / alpha, (1. / 3.)) + 1. / (alpha * 9.) - 1.);
       gama = alnorm (pn1, FALSE);
       return gama;
     }
@@ -856,8 +932,7 @@ polygamma (long n, double z)
 /* syntax: nrcheck(matrix,inversematrix,ncols=nrows,returnval1,returnval2) */
 /* mai 95 PB                                             */
 boolean
-nrcheck (double **m, double **tm, double *v, long nrows, double *r1,
-	 double *r2, boolean do_newton)
+nrcheck (double **m, double **tm, double *v, long nrows, double *r1, double *r2, boolean do_newton)
 {
   long i, j, k;
   double *tmp, *tmp2, tmp3 = 0.0, tmp4 = 0.0;
@@ -1057,33 +1132,33 @@ alnorm (double x, int up)
 {
   /* Initialized data */
   /* *** machine dependent constants ????????????? */
-  /*static*/ double zero = 0.;
-  /*static*/ double a1 = 5.75885480458;
-  /*static*/ double a2 = 2.62433121679;
-  /*static*/ double a3 = 5.92885724438;
-  /*static*/ double b1 = -29.8213557807;
-  /*static*/ double b2 = 48.6959930692;
-  /*static*/ double c1 = -3.8052e-8;
-  /*static*/ double c2 = 3.98064794e-4;
-  /*static*/ double c3 = -.151679116635;
-  /*static*/ double c4 = 4.8385912808;
-  /*static*/ double c5 = .742380924027;
-  /*static*/ double one = 1.;
-  /*static*/ double c6 = 3.99019417011;
-  /*static*/ double d1 = 1.00000615302;
-  /*static*/ double d2 = 1.98615381364;
-  /*static*/ double d3 = 5.29330324926;
-  /*static*/ double d4 = -15.1508972451;
-  /*static*/ double d5 = 30.789933034;
-  /*static*/ double half = .5;
-  /*static*/ double ltone = 7.;
-  /*static*/ double utzero = 18.66;
-  /*static*/ double con = 1.28;
-  /*static*/ double p = .398942280444;
-  /*static*/ double q = .39990348504;
-  /*static*/ double r = .398942280385;
+  /*static */ double zero = 0.;
+  /*static */ double a1 = 5.75885480458;
+  /*static */ double a2 = 2.62433121679;
+  /*static */ double a3 = 5.92885724438;
+  /*static */ double b1 = -29.8213557807;
+  /*static */ double b2 = 48.6959930692;
+  /*static */ double c1 = -3.8052e-8;
+  /*static */ double c2 = 3.98064794e-4;
+  /*static */ double c3 = -.151679116635;
+  /*static */ double c4 = 4.8385912808;
+  /*static */ double c5 = .742380924027;
+  /*static */ double one = 1.;
+  /*static */ double c6 = 3.99019417011;
+  /*static */ double d1 = 1.00000615302;
+  /*static */ double d2 = 1.98615381364;
+  /*static */ double d3 = 5.29330324926;
+  /*static */ double d4 = -15.1508972451;
+  /*static */ double d5 = 30.789933034;
+  /*static */ double half = .5;
+  /*static */ double ltone = 7.;
+  /*static */ double utzero = 18.66;
+  /*static */ double con = 1.28;
+  /*static */ double p = .398942280444;
+  /*static */ double q = .39990348504;
+  /*static */ double r = .398942280385;
 
-  /*static*/ double y, result;
+  /*static */ double y, result;
 
   if (x < zero)
     {
@@ -1095,16 +1170,10 @@ alnorm (double x, int up)
       y = half * x * x;
       if (x > con)
 	{
-	  result =
-	    r * exp (-y) / (x + c1 +
-			    d1 / (x + c2 +
-				  d2 / (x + c3 +
-					d3 / (x + c4 +
-					      d4 / (x + c5 + d5 / (x + c6))))));
+	  result = r * exp (-y) / (x + c1 + d1 / (x + c2 + d2 / (x + c3 + d3 / (x + c4 + d4 / (x + c5 + d5 / (x + c6))))));
 	  return ((!up) ? one - result : result);
 	}
-      result =
-	half - x * (p - q * y / (y + a1 + b1 / (y + a2 + b2 / (y + a3))));
+      result = half - x * (p - q * y / (y + a1 + b1 / (y + a2 + b2 / (y + a3))));
       return ((!up) ? one - result : result);
     }
   else
@@ -1248,7 +1317,7 @@ d1mach (long i)
     case 4:
       return DBL_EPSILON;
     case 5:
-      return log10 (FLT_RADIX);
+      return log10 ((double) FLT_RADIX);
     }
   usererror ("invalid argument: d1mach(%ld)\n", i);
   return 0;			/* for compilers that complain of missing return values */
@@ -1297,13 +1366,12 @@ i1mach (long i)
 }
 
 int
-dpsifn (double *x, long *n, long kode, long m, double *ans, long *nz,
-	long *ierr)
+dpsifn (double *x, long *n, long kode, long m, double *ans, long *nz, long *ierr)
 {
   /* Initialized data */
 
-  /*static*/ long nmax = 100;
-  /*static*/ double b[22] = { 1., -.5, .166666666666666667,
+  /*static */ long nmax = 100;
+  /*static */ double b[22] = { 1., -.5, .166666666666666667,
     -.0333333333333333333, .0238095238095238095, -.0333333333333333333,
     .0757575757575757576, -.253113553113553114, 1.16666666666666667,
     -7.09215686274509804, 54.9711779448621554, -529.124242424242424,
@@ -1319,17 +1387,16 @@ dpsifn (double *x, long *n, long kode, long m, double *ans, long *nz,
 
 
   /* Local variables */
-  /*static*/ double elim, xinc, xmin, tols, xdmy, yint, trmr[100], rxsq;
-  /*static*/ long i__, j, k;
-  /*static*/ double s, t, slope, xdmln, wdtol;
-  /*static*/ double t1, t2;
-  /*static*/ long fn;
-  /*static*/ double ta;
-  /*static*/ long mm, nn, np;
-  /*static*/ double fx, tk;
-  /*static*/ long mx, nx;
-  /*static*/ double xm, tt, xq, den, arg, fln, r1m4, r1m5, eps, rln, tol, xln,
-    trm[22], tss, tst;
+  /*static */ double elim, xinc, xmin, tols, xdmy, yint, trmr[100], rxsq;
+  /*static */ long i__, j, k;
+  /*static */ double s, t, slope, xdmln, wdtol;
+  /*static */ double t1, t2;
+  /*static */ long fn;
+  /*static */ double ta;
+  /*static */ long mm, nn, np;
+  /*static */ double fx, tk;
+  /*static */ long mx, nx;
+  /*static */ double xm, tt, xq, den, arg, fln, r1m4, r1m5, eps, rln, tol, xln, trm[22], tss, tst;
 
   /* Parameter adjustments */
   --ans;
@@ -1502,7 +1569,7 @@ L80:
   for (i__ = 1; i__ <= i1; ++i__)
     {
       i2 = -np;
-      trmr[i__ - 1] = pow (fx, i2);
+      trmr[i__ - 1] = pow (fx, (double) i2);
       s += trmr[i__ - 1];
       xm += -1.;
       fx = *x + xm;
@@ -1620,7 +1687,7 @@ L200:
     {
       den += 1.;
       i2 = -np;
-      trm[i__ - 1] = pow (den, i2);
+      trm[i__ - 1] = pow (den, (double) i2);
       s += trm[i__ - 1];
 /* L210: */
     }
@@ -1671,7 +1738,7 @@ L220:
 /* ----------------------------------------------------------------------- */
 L260:
   i1 = -(*n) - 1;
-  ans[1] = pow (*x, i1);
+  ans[1] = pow (*x, (double) i1);
   if (mm == 1)
     {
       goto L280;
@@ -1778,7 +1845,7 @@ double
 probchi (long df, double chi)
 {
   double prob;
-  double v = ((double) df)/2.;
+  double v = ((double) df) / 2.;
   if (chi > DBL_EPSILON)
     {
       //lg = exp (LGAMMA (v));
@@ -1787,18 +1854,16 @@ probchi (long df, double chi)
   else
     prob = 1.0;
   //  printf("prob=%f v=%f chi=%f lg(v/2)=%f  ig(chi/2,v/2)=%f\n",
-   //  prob,v,chi,lg, incompletegamma(chi/2.,v/2.));
-   
+  //  prob,v,chi,lg, incompletegamma(chi/2.,v/2.));
+
   return prob;
 }
 
 double
 chisquare (long df, double alpha)
 {
-  const double table05[] =
-    { 3.84146, 5.99147, 7.81473, 9.48773, 11.0705, 12.5916 };
-  const double table01[] =
-    { 6.63490, 9.21034, 11.3449, 13.2767, 15.0863, 16.8119 };
+  const double table05[] = { 3.84146, 5.99147, 7.81473, 9.48773, 11.0705, 12.5916 };
+  const double table01[] = { 6.63490, 9.21034, 11.3449, 13.2767, 15.0863, 16.8119 };
 
   if (alpha == 0.05)
     return table05[df - 1];
@@ -1840,6 +1905,7 @@ mstart (long pop, long numpop)
 {
   return numpop + pop * numpop - pop;
 }
+
 // returns the end of  mig_.i
 long
 mend (long pop, long numpop)
@@ -1859,6 +1925,7 @@ mmend (long pop, long numpop)
 {
   return pop * numpop + numpop;
 }
+
 // returns the location in a full matrix given the abbreviated matrix
 long
 mm2m (long frompop, long topop, long numpop)
@@ -1954,14 +2021,14 @@ gamma_rates (double *rate, long categs, char *input)
     alpha = value;
   calc_gamma (alpha, rate, categs);
 }
+
 /* calculation of rate values following a gamma distribution for
    given probability values */
 void
 calc_gamma (double alpha, double *gama, long categs)
 {
   long i, panic;
-  double low, mid, high, xlow, xhigh, tmp, freq = 0, x = 10,
-    elements = (double) categs;
+  double low, mid, high, xlow, xhigh, tmp, freq = 0, x = 10, elements = (double) categs;
   freq = -(0.5 / elements);	/*so we have midpoints instead of endpoints */
   for (i = 0; i < categs; i++)
     {
@@ -2014,3 +2081,64 @@ calc_gamma (double alpha, double *gama, long categs)
 }
 
 
+void
+print_line (FILE * outfile, char c, long nn, long flag)
+{
+  long i, start = 0;
+  switch (flag)
+    {
+    case START:
+      start = 2;
+      fputc ('=', outfile);
+      fputc ('-', outfile);
+      fputc ('-', outfile);
+      break;
+    case STOP:
+      start = 2;
+      fputc ('=', outfile);
+      fputc ('=', outfile);
+      fputc ('-', outfile);
+      break;
+    default:
+      start = 0;
+    }
+  for (i = start; i < nn; i++)
+    {
+      fputc (c, outfile);
+    }
+  fputc ('\n', outfile);
+}
+
+char sgetc(char **buffer)
+{
+  char ch;
+  ch = **buffer;
+  (*buffer)++;
+  return ch;
+}
+
+
+char *sgets(char *s, int size, char **stream)
+{
+  long ch='\0';
+  long counter = 0;
+  while(counter < size-1)
+    {
+      ch = **stream;
+      (*stream)++;
+      switch(ch)
+	{
+	case '\n':
+	  s[counter]='\0';
+	  return s;
+	default:
+	  s[counter] = ch;
+	  break;
+	}
+      counter++;
+    }
+  return s;
+}
+
+
+
